<?php

	declare(strict_types=1);
	
	namespace DATADIAGRAMSPRO;
	
	if (count(get_included_files()) < 2) { header("HTTP/1.1 301 Moved Permanently"); header("Location: /"); exit; } 

	class SchemaValidator
	{
				
		private $validationErrors = "";	
		
		private $XSDNamespace = "http://www.w3.org/2001/XMLSchema";

		
		private ?object $log = null;

		public function __construct(object $iLog = null)
		{
			$this->log = $iLog;	
		}
		
		public function validate(\DomDocument $iXMLDoc, string $iXMLSchemaURL) : bool  // \DomDocument
		{
			try
			{									
				// Enable user error handling
				libxml_use_internal_errors(true);
		
				if( !$iXMLDoc->schemaValidate($iXMLSchemaURL) ) 
				{
					$this->validationErrors = $this->registerErrors();
				
					return false;
				} 
				else 
				{
					return true;
				}
			
			}
			catch(\Throwable |  \DOMException $e)
			{	
				throw $e;	
						
				return false;
			}	
		}
		
		public function getErrors() 
		{
		
			if($this->log)
			{				
				$this->log->trace(__CLASS__, __FUNCTION__);
			}	

			// remove {http://www.w3.org/2001/XMLSchema} for shorter message
			// TODO: should I remove others ?
			return $this->validationErrors;
		}
		
		private function registerError($error)
		{
			try
			{		
				if($this->log)
				{				
					$this->log->trace(__CLASS__, __FUNCTION__);
				}	
						
				$result = "";

				switch ($error->level) 
				{
					case LIBXML_ERR_WARNING:
						$result .= "Validation warning"; //  $error->code: 
						break;
					case LIBXML_ERR_ERROR:
						$result .= "Validation error";
						break;
					case LIBXML_ERR_FATAL:
						$result .= "Fatal validation error";
						break;
				}

				$result .= " on line " . $error->line . ": ";
		
				// remove XMLSchema namespace - for shorter message
				// TODO: should I remove others ?
				$result .= str_replace("{". $this->XSDNamespace ."}", "", $error->message);

				$result = trim($result);
	
				return $result;
			
			}
			catch(\Throwable $e)
			{
				throw $e;				
				
				return false;
			}							
		}
	
		private function registerErrors() 
		{
			try
			{		
				if($this->log)
				{				
					$this->log->trace(__CLASS__, __FUNCTION__);
				}	
						
				$result = "";
				$errors = libxml_get_errors();
		
				$highestLevel = 0;
		
				foreach ($errors as $error) 
				{
					if ($error->level > $highestLevel) 
					{
						$highestLevel = $error->level;
						//print_r($error);
						$result =  $this->registerError($error);
					}
				}
		
				libxml_clear_errors();

				return $result;
				
			}
			catch(\Throwable $e)
			{
				throw $e;				
				
				return false;
			}	
		}
			
	}

?>