// Javascript

'use strict';
	
	new DataDiagrams();	

	function DataDiagrams()
	{
		this.url = 'https://data-diagrams.com/api.php';

		const cSelf =  this;	

		document.addEventListener('DOMContentLoaded', function() {
			
			try
			{										
				const iframeArr = document.querySelectorAll("iframe");

				iframeArr.forEach( (item) => {

					item.addEventListener("load", (e) => {
						
						document.getElementById("loading").setAttribute('hidden', '');			
	
					});

				})		
			
			}
			catch(e)
			{
				_errorHandler(e);			
			}			
	
		});
	
		window.addEventListener("message", function (e) {
			
			try
			{				
				const data = e.data;
				
				const JData = JSON.parse(data);	
																																
				const content = JData.content;
			
			
				if(content === false)				
				{// FREE
								
					const sessionID = JData.session;
																		
					addMedia(sessionID, 'Free Diagram', '');
			
				}
				else
				{// PRO
				
					saveSVGDiagrams(content);
								
				}					
			
			}
			catch(e)
			{
				_errorHandler(e);			
			}	
		});


	// media library
		function addMedia(sessionID, title, caption)
		{
			try
			{		
				const url = cSelf.url + '?id=' + sessionID;
		
				getSVG(url, title, caption); 
	
			}
			catch(e)
			{
				_errorHandler(e);			
			}	
		}
	
		function getSVG(url, title, caption)
		{// get SVG from data-diagrams API
			try
			{
				fetch(url, {
					mode: 'cors',
					})
					.then(function(response) {
						return response.blob()
					})
					.then(function(blob) {

						// upload to media library				
						const urlx = wpApiSettings.root + 'wp/v2/media'; // *
	
						sendMedia(urlx, blob, title, caption);
			
					})
					.catch(error => {
						console.error('Error:', error); // check loggen, ved fejl
					});
				
			}
			catch(e)
			{
				_errorHandler(e);			
			}	
		}
		
		function sendMedia(url, blob, title, caption)
		{ // upload SVG to media library
			try
			{	
				var file = new File([blob], "data-diagram.svg");
	
				const formData = new FormData();
	
				formData.append("file", file);					
					
				formData.append("title", title);
				formData.append("caption", caption);
				formData.append("alternative", "data-diagrams.com");
				formData.append("description", "http://data-diagrams.com");
			
				fetchPOST(url, formData);
		
			}
			catch(e)
			{
				_errorHandler(e);			
			}	
		}

	
	// save files to disk
		// async function saveSVGDiagrams(url, formData)
		async function saveSVGDiagrams(content)
		{
			try
			{			
				const formData = new FormData(); 
		
				content.forEach( (item) => {
			
					const id = item.id;
					const name = item.name;
				
					const svg = atob(item.svg);												
					const xsl = atob(item.xsl);
																														
					formData.append('data[' + id + '][svg]', svg);
					formData.append('data[' + id + '][xsl]', xsl);
																		
				})
		
				const url = wpApiSettings.url + 'REST.php';
									
				fetchPOST(url, formData, true);
				
			}
			catch(e)
			{
				_errorHandler(e);			
			}	
		}
		
		
	// shared	
		function fetchPOST(url, formData, hasReturn)
		{
			try
			{			
				const nonce = wpApiSettings.nonce;

				fetch(url, {			
					method: "POST",
					headers: {
						'X-WP-Nonce': nonce,
					},
				  body: formData
				})
				.then(response => response.json())
				.catch(error => {

					console.error('Error:', error); // check loggen, when error
				});
			
			}
			catch(e)
			{
				_errorHandler(e);			
			}		
		}
		
		function _errorHandler(e)
		{
			console.error(e);		
		}
			
	}